<?php
function nexa_enqueue_styles() {
    $stylesheet_uri = get_template_directory_uri() . '/style.css';
    $version = filemtime(get_template_directory() . '/style.css');
    wp_register_script('hamburger', get_template_directory_uri() . '/hamburger.js', array('jquery'), null, true);
    wp_enqueue_script('hamburger');
    wp_enqueue_script('skip-link-focus-fix', get_template_directory_uri() . '/skip-link-focus-fix.js', array(), '1.0', true);
    wp_enqueue_style('nexa-theme-style', $stylesheet_uri, array(), $version);
}

add_action('wp_enqueue_scripts', 'nexa_enqueue_styles');

function nexa_register_menus() {
    register_nav_menus(
        array(
            'primary' => __('Primary Menu', 'nexa'),
            'secondary' => __('Secondary Menu', 'nexa')
        )
    );

    $dimensions = array(
        'width' => 1920,
        'height' => 250
    );

    add_theme_support('post-thumbnails');
    add_theme_support('custom-header', $dimensions);
    add_theme_support('title-tag');
    add_theme_support( 'automatic-feed-links' );
    add_theme_support("wp-block-styles");
    add_theme_support("responsive-embeds");
    add_theme_support("html5", array(
        "search-form",
        "gallery",
        "caption"
    ));
    add_theme_support("custom-logo", array(
        "height" => 100,
        "width" => 400,
        "flex-height" => true,
        "flex-width" => true
    ));
    add_theme_support("custom-background", array(
        "default-color" => "ffffff",
        "default-image" => "",
        "default-repeat" => "repeat",
        "default-position-x" => "left",
        "default-position-y" => "top",
        "default-size" => "auto",
        "default-attachment" => "scroll"
    ));
    add_theme_support("align-wide");
}

function nexa_editor_styles() {
    add_editor_style('style.css');
}
add_action('admin_init', 'nexa_editor_styles');

function nexa_comments_reply() {
    if( get_option( 'thread_comments' ) )  {
        wp_enqueue_script( 'comment-reply' );
    }
}
add_action( 'comment_form_before', 'nexa_comments_reply' );


add_action('after_setup_theme', 'nexa_register_menus');

function nexa_sidebar() {
    register_sidebar(
        array(
            'name' => __('Blog Sidebar', 'nexa'),
            'id' => 'blog-sidebar',
            'description' => __('Blog Sidebar', 'nexa'),
            'before_widget' => '<div class="demo-theme-sidebar">',
            'after_widget' => '</div>',
            'before_title' => '<h3 class="demo-theme-sidebar-title>',
            'after_title' => '</h3>'
        )
    );
}
add_action('widgets_init', 'nexa_sidebar');

function nexa_custom_excerpt_length() {
    return 20;
}
add_filter('excerpt_length', 'nexa_custom_excerpt_length', 999);


function nexa_customize_register( $wp_customize ) {
    $wp_customize->add_section( 'nexa_colors' , array(
        'title'      => esc_html__( 'Nexa Colors', 'nexa' ),
        'priority'   => 30,
    ) );

    $colors = array(
        'nexa_primary_color' => array('default' => '#ff9900', 'label' => 'Primary Color'),
        'nexa_secondary_color' => array('default' => '#ffcc66', 'label' => 'Secondary Color'),
        'nexa_header_footer_bg_color' => array('default' => '#333', 'label' => 'Header/Footer Background Color'),
        'nexa_text_color' => array('default' => '#dfdfdf', 'label' => 'Text Color'),
        'nexa_borders_color' => array('default' => '#444', 'label' => 'Borders Color'),
        'nexa_background_color' => array('default' => '#1d1d1d', 'label' => 'Background Color')
    );

    foreach ($colors as $id => $value) {
        $wp_customize->add_setting( $id , array(
            'default'           => $value['default'],
            'transport'         => 'postMessage',
            'sanitize_callback' => 'sanitize_hex_color',
        ) );
        $wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, $id, array(
            'label'      => esc_html($value['label']),
            'section'    => 'nexa_colors',
            'settings'   => $id,
        ) ) );
    }

    $wp_customize->add_setting('hero_background_image', array(
        'default' => '',
        'transport' => 'refresh',
        'sanitize_callback' => 'esc_url_raw'
    ));
    
    
    $wp_customize->add_control(new WP_Customize_Image_Control($wp_customize, 'hero_background_image', array(
        'label' => __('Hero Background Image', 'nexa'),
        'section' => 'title_tagline',
        'settings' => 'hero_background_image',
    )));

    
}
add_action( 'customize_register', 'nexa_customize_register' );

function nexa_customizer_live_preview() {
    wp_enqueue_script( 'nexa-themecustomizer', esc_url(get_template_directory_uri().'/customizer.js'), array( 'jquery', 'customize-preview' ), '', true );
}
add_action( 'customize_preview_init', 'nexa_customizer_live_preview' );


function nexa_customizer_css() {
    ?>
    <style type="text/css">
        :root {
            --primary: <?php echo esc_attr(get_theme_mod('nexa_primary_color', '#ff9900')); ?>;
            --secondary: <?php echo esc_attr(get_theme_mod('nexa_secondary_color', '#ffcc66')); ?>;
            --header-footer-bg-color: <?php echo esc_attr(get_theme_mod('nexa_header_footer_bg_color', '#333')); ?>;
            --text: <?php echo esc_attr(get_theme_mod('nexa_text_color', '#dfdfdf')); ?>;
            --borders: <?php echo esc_attr(get_theme_mod('nexa_borders_color', '#444')); ?>;
            --background: <?php echo esc_attr(get_theme_mod('nexa_background_color', '#1d1d1d')); ?>;
        }
    </style>
    <?php
}
add_action( 'wp_head', 'nexa_customizer_css');

function nexa_skip_link() {
	echo '<a class="skip-link screen-reader-text" href="#main-content">' .
		__( 'Skip to the content', 'nexa' ) .
	'</a>';
}

add_action( 'wp_body_open', 'nexa_skip_link', 5 );

function nexa_default_menu() {
    get_template_part('sections/default-menu');
}


function nexa_customize_about_author( $wp_customize ) {
    
    $wp_customize->add_section( 'nexa_about_author' , array(
        'title'      => __( 'About the Author', 'nexa' ),
        'priority'   => 1, 
    ) );

    $wp_customize->add_control( new WP_Customize_Control(
        $wp_customize,
        'nexa_author_description',
        array(
            'label'          => __( 'Author Information', 'nexa' ),
            'section'        => 'nexa_about_author',
            'settings'       => array(), 
            'type'           => 'hidden', 
            'description'    => 'Hello! I am Josef Salazar, the author of Nexa. Thank you for using my WordPress theme. If you have any questions or feedback, please reach out. Visit my website at www.josefsalazar.com for more information.',
            'priority'       => 1,
        )
    ) );


}
add_action( 'customize_register', 'nexa_customize_about_author' );